# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-15 09:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cerberus_ac', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='privilegehistory',
            name='authorized',
            field=models.NullBooleanField(default=None, verbose_name='Authorized'),
        ),
    ]
